import sys
import os
from loguru import logger
from typing import List, Dict

here = os.path.dirname(__file__)

sys.path.append(os.path.join(here, ".."))

from .raw_to_bronze_pipeline import RawToBronzePipeline  # noqa: E402


class RawToBronzeMetadataPipeline(RawToBronzePipeline):
    INPUT_KEY = "metadata"
    OUTPUT_KEY = "metadata"

    def transform(self, source_data: List[Dict[str, str]]):
        """
        Transforms the data.

        :param source_data: The data to transform.
        :return: The transformed data.
        """
        logger.info("Transforming data.")
        return [
            (acc_id, self.format_metadata(data))
            for acc_id, data_object in source_data
            for key, data in data_object
        ]

    def format_metadata(self, metadata):
        """
        Formats the metadata.

        :param metadata: The metadata to format.
        :return: The formatted metadata.
        """
        return metadata



from abc import ABC, abstractmethod
from typing import Dict


class DataLoader(ABC):
    @abstractmethod
    def read(self):
        """
        Base method for loading data.
        """
        raise NotImplementedError()

    @abstractmethod
    def write(self):
        """
        Base method for writing data.
        """
        raise NotImplementedError()

    def build_partition_path(self, partition_config: Dict[str, str]):
        partition_id = ""
        for partition_name, partition_value in partition_config.items():
            partition_id += "{0}={1}/".format(partition_name, partition_value)
        return partition_id[:-1]



import boto3
from botocore.exceptions import ClientError
from .connection import Connection
import os
from loguru import logger


class AWSConnection(Connection):
    def __init__(self) -> None:
        super().__init__()
        self.session: boto3.session.Session = AWSConnection.connect(self)

    def connect(self) -> boto3.session.Session:
        """
        Base method for connecting to the data source.

        :return: The session object.
        """
        logger.info("Connecting to AWS session...")
        return self._get_session()

    def get_aws_secret(self, secret_name: str) -> str:
        """
        Method to retrieve a secret from AWS Secrets Manager.

        :param secret_name: The name of the secret to retrieve.

        :return: The secret value.
        """
        client: boto3.client = self.session.client(
            service_name="secretsmanager",
            region_name="us-east-1",
        )
        logger.info(
            f"Retrieving secret {secret_name} from AWS Secrets Manager..."
        )  # noqa
        try:
            get_secret_value_response = client.get_secret_value(
                SecretId=secret_name
            )
        except ClientError as e:
            raise e

        secret = get_secret_value_response["SecretString"]
        return secret

    def _get_session(self) -> boto3.session.Session:
        """
        Method to retrieve the session object.

        :return: The session object.
        """
        if os.environ.get("AWS_LAMBDA_FUNCTION_NAME"):
            session = boto3.session.Session()
        else:
            session = boto3.session.Session(
                aws_access_key_id=os.environ.get("AWS_ACCESS_KEY_ID"),
                aws_secret_access_key=os.environ.get("AWS_SECRET_ACCESS_KEY"),
                region_name=os.environ.get("AWS_REGION_NAME"),
            )
        return session



from aws_cdk import Stack
from aws_cdk import aws_s3 as _s3
from constructs import Construct


class S3DeployStack(Stack):
    def __init__(
        self, scope: Construct, construct_id: str, bucket_id: str, **kwargs
    ) -> None:
        super().__init__(scope, construct_id, **kwargs)
        self.create_bucket(bucket_id)

    def create_bucket(self, bucket_id: str):
        _s3.Bucket(
            self,
            bucket_id,
            bucket_name=bucket_id,
        )



from budgetguard.main import run
from loguru import logger


def lambda_handler(event, context):
    logger.info("Starting raw to bronze balances pipeline lambda..")
    task = "raw_to_bronze_balances"
    partition_id = event["partition_id"]
    logger.info(f"Running task: {task} for partition: {partition_id}")
    run(task, partition_id)
    logger.info(f"Finished running task: {task} for partition: {partition_id}")



import requests
from uuid import uuid4
import argparse
from nordigen import NordigenClient

parser = argparse.ArgumentParser()

parser.add_argument(
    "-sid",
    "--secret_id",
    help="Nordigen secret id",
    type=str,
    required=True,
)

parser.add_argument(
    "-skey",
    "--secret_key",
    help="Nordigen secret key",
    type=str,
    required=True,
)

if __name__ == "__main__":
    args = parser.parse_args()
    client = NordigenClient(
        secret_id=args.secret_id,
        secret_key=args.secret_key,
    )
    token_data = client.generate_token()["access"]

    url = "https://bankaccountdata.gocardless.com/api/v2/agreements/enduser/"
    headers = {
        "accept": "application/json",
        "Content-Type": "application/json",
        "Authorization": f"Bearer {token_data}",
    }

    data = {
        "institution_id": "BANK_MILLENNIUM_BIGBPLPW",
        "max_historical_days": "90",
        "access_valid_for_days": "90",
        "access_scope": ["balances", "details", "transactions"],
    }

    response = requests.post(url, headers=headers, json=data)

    print(response.status_code)
    print(response.json())

    url = "https://bankaccountdata.gocardless.com/api/v2/requisitions/"
    headers = {
        "accept": "application/json",
        "Content-Type": "application/json",
        "Authorization": f"Bearer {token_data}",
    }

    data = {
        "redirect": "http://www.yourwebpage.com",
        "institution_id": "BANK_MILLENNIUM_BIGBPLPW",
        "reference": str(uuid4()),
        "agreement": response.json()["id"],
        "user_language": "EN",
    }

    response = requests.post(url, headers=headers, json=data)

    print(response.status_code)
    print(response.json())



import sys
import os
from loguru import logger
from typing import List, Dict

here = os.path.dirname(__file__)

sys.path.append(os.path.join(here, ".."))

from .raw_to_bronze_pipeline import RawToBronzePipeline  # noqa: E402


class RawToBronzeDetailsPipeline(RawToBronzePipeline):
    INPUT_KEY = "details"
    OUTPUT_KEY = "details"

    def transform(self, source_data: List[Dict[str, str]]):
        """
        Transforms the data.

        :param source_data: The data to transform.
        :return: The transformed data.
        """
        logger.info("Transforming data.")
        return [
            (acc_id, self.format_details(data))
            for acc_id, data_object in source_data
            for key, data in data_object
        ]

    def format_details(self, details):
        """
        Formats the details.

        :param details: The details to format.
        :return: The formatted details.
        """
        details_flattened = details.get("account", {})
        details_flattened = {
            self.camel_to_snake(k): v for k, v in details_flattened.items()
        }
        return details_flattened



import sys
import os
from loguru import logger
from typing import List, Dict

here = os.path.dirname(__file__)

sys.path.append(os.path.join(here, ".."))

from .raw_to_bronze_pipeline import RawToBronzePipeline  # noqa: E402


class RawToBronzeTransactionsPipeline(RawToBronzePipeline):
    INPUT_KEY = "transactions"
    OUTPUT_KEY = "transactions"

    def transform(self, source_data: List[Dict[str, str]]):
        """
        Transforms the data.

        :param source_data: The data to transform.
        :return: The transformed data.
        """
        logger.info("Transforming data.")
        return [
            (acc_id, self.format_transactions(data))
            for acc_id, data_object in source_data
            for key, data in data_object
        ]

    def format_transactions(self, transactions):
        """
        Formats the transactions.

        :param transactions: The transactions to format.
        :return: The formatted transactions.
        """
        transactions_flatten = []
        logger.info("Flattening transactions...")
        for transaction_status, transactions_info in transactions[
            "transactions"
        ].items():
            logger.info(
                "Flattening transactions for status {0}...".format(
                    transaction_status
                )
            )  # noqa
            logger.info(
                "Number of transactions with status {0}: {1}".format(
                    transaction_status, len(transactions_info)
                )
            )
            for transaction in transactions_info:
                transaction["transactionStatus"] = transaction_status
                transaction["transactionCurrency"] = transaction[
                    "transactionAmount"
                ]["currency"]
                transaction["transactionAmount"] = transaction[
                    "transactionAmount"
                ]["amount"]
                if transaction.get("debtorAccount"):
                    transaction["debtorAccountIban"] = transaction[
                        "debtorAccount"
                    ]["iban"]
                    del transaction["debtorAccount"]
                else:
                    transaction["creditorAccountIban"] = transaction[
                        "creditorAccount"
                    ]["iban"]
                    del transaction["creditorAccount"]
                transaction["balanceAfterTransactionAmount"] = transaction[
                    "balanceAfterTransaction"
                ]["balanceAmount"]["amount"]
                transaction["balanceAfterTransactionCurrency"] = transaction[
                    "balanceAfterTransaction"
                ]["balanceAmount"]["currency"]
                del transaction["remittanceInformationUnstructured"]
                del transaction["balanceAfterTransaction"]
                transactions_flatten.append(
                    {self.camel_to_snake(k): v for k, v in transaction.items()}
                )
        logger.info("Finished flattening transactions!")
        return transactions_flatten



from abc import ABC, abstractmethod


class Pipeline(ABC):
    INPUT_DATA_LOADER = "input_data_loader"
    OUTPUT_DATA_LOADER = "output_data_loader"
    INPUT_DATA_CONFIG = {}

    @abstractmethod
    def run(self):
        """
        Base method for running the pipeline.
        """
        raise NotImplementedError()

    @abstractmethod
    def read_sources(self):
        """
        Base method for reading data from the data sources.
        """
        raise NotImplementedError()

    @abstractmethod
    def write_sources(self):
        """
        Base method for writing data to the data sources.
        """
        raise NotImplementedError()

    @abstractmethod
    def transform(self):
        """
        Base method for transforming the data.
        """
        raise NotImplementedError()



from budgetguard.main import run
from loguru import logger
from datetime import datetime
from datetime import timedelta


def lambda_handler(event, context):
    logger.info("Starting ingestion lambda..")
    task = "ingest_account_data"
    yesterday = datetime.now() - timedelta(days=1)
    partition_id = yesterday.strftime("%Y%m%d")
    logger.info(f"Running task: {task} for partition: {partition_id}")
    run(task, partition_id)
    response = {"partition_id": partition_id}
    logger.info(f"Finished running task: {task} for partition: {partition_id}")
    return response



from .nordigen_data_loader import NordigenDataLoader
from .s3_data_loader import S3DataLoader
from .data_loader import DataLoader
from .spark_s3_data_loader import SparkS3DataLoader
from .exchange_rates_data_loader import ExchangeRatesDataLoader


def create_data_loader(data_loader_type: str) -> DataLoader:
    """
    Factory function for creating data loaders.

    :param data_loader_type: The type of data loader to create.

    :return: The data loader.
    """
    if data_loader_type == "nordigen":
        return NordigenDataLoader()
    elif data_loader_type == "s3":
        return S3DataLoader()
    elif data_loader_type == "spark_s3":
        return SparkS3DataLoader()
    elif data_loader_type == "exchange_rates":
        return ExchangeRatesDataLoader()
    else:
        raise ValueError(f"Data loader type {data_loader_type} not supported.")



from .connection import Connection
from .s3_connection import S3Connection
from pyspark.sql import SparkSession
from pyspark.conf import SparkConf
from loguru import logger
import json


class SparkS3Connection(Connection):
    NAME = "spark_s3"

    def __init__(self) -> None:
        super().__init__()
        self.s3_connection: S3Connection = S3Connection()
        self.spark_session: SparkSession = SparkS3Connection.connect(self)

    def __create_spark_session__(self) -> SparkSession:
        if self.platform == self.__Platform__.LOCAL:
            conf = SparkConf()
            conf.set("spark.logConf", "true")
            spark = (
                SparkSession.builder.appName("BudgetGuard")
                .config(
                    "spark.jars",
                    "/opt/miniconda3/lib/python3.8/site-packages/pyspark/jars/aws-java-sdk-bundle-1.11.901.jar,/opt/miniconda3/lib/python3.8/site-packages/pyspark/jars/hadoop-aws-3.3.1.jar",  # noqa: E501
                )
                .getOrCreate()
            )
            spark = self.__build_local_spark_conf__(spark)
            spark.sparkContext.setLogLevel("ERROR")
        elif self.platform == self.__Platform__.EMR:
            spark = SparkSession.builder.appName("BudgetGuard").getOrCreate()
        else:
            raise Exception("Unknown platform!")
        return spark

    def connect(self) -> SparkSession:
        logger.info("Connecting to Spark session...")
        return self.__create_spark_session__()

    def __build_local_spark_conf__(self, spark: SparkSession) -> SparkSession:
        aws_credentials = json.loads(
            self.s3_connection.get_aws_secret("budget_guard_aws_credentials")
        )
        spark._jsc.hadoopConfiguration().set(
            "fs.s3a.access.key", aws_credentials["aws_access_key_id"]
        )
        spark._jsc.hadoopConfiguration().set(
            "fs.s3a.secret.key", aws_credentials["aws_secret_access_key"]
        )
        spark._jsc.hadoopConfiguration().set(
            "fs.s3a.impl", "org.apache.hadoop.fs.s3a.S3AFileSystem"
        )
        spark._jsc.hadoopConfiguration().set(
            "com.amazonaws.services.s3.enableV4", "true"
        )
        spark._jsc.hadoopConfiguration().set(
            "fs.s3a.aws.credentials.provider",
            "org.apache.hadoop.fs.s3a.SimpleAWSCredentialsProvider",
        )
        spark._jsc.hadoopConfiguration().set(
            "fs.s3a.multiobjectdelete.enable", "false"
        )
        return spark
